import sys
import os
from platform import python_version

# https://github.com/willmcgugan/rich
from rich.console import Console
from rich.theme import Theme

import pydbhub.dbhub as dbhub


if __name__ == '__main__':
    custom_theme = Theme({
        "info": "green",
        "warning": "yellow",
        "error": "bold red"
    })
    console = Console(theme=custom_theme)

    if python_version()[0:3] < '3.7':
        console.print("[ERROR] Make sure you have Python 3.7+ installed, quitting.\n\n", style="error")
        sys.exit(1)

    # Create a new DBHub.io API object
    db = dbhub.Dbhub(config_file=f"{os.path.join(os.path.dirname(__file__), '..', 'config.ini')}")

    # Retrieve the metadata for the remote database
    meta, err = db.Metadata("justinclift", "Join Testing.sqlite")
    if err is not None:
        console.print(f"[ERROR] {err}", style="error")
        sys.exit(1)

    # * Display the retrieved metadata *

    # Display the database branches
    console.print('Branches:', style="info")
    for branche_name in meta.branches:
        console.print(f"   - {branche_name} #commit: {meta.branches[branche_name].commit_count}", style="info")
    console.print(f"Default branch: {meta.default_branch}", style="info")

    # Display the database releases
    if len(meta.releases) > 0:
        console.print('Releases:', style="info")
        for release in meta.releases:
            console.print(f"   - {release}", style="info")

    # Display the database tags
    if len(meta.tags) > 0:
        console.print('Tags:', style="info")
        for tag in meta.tags:
            console.print(f"   - {tag}", style="info")

    # Display the database commits
    if len(meta.commits) > 0:
        console.print('Commits:', style="info")
        for commit in meta.commits:
            console.print(f"   - {commit.id}, {commit.timestamp}", style="info")

    # Display the web page for the database
    console.print(f"Web page: {meta.web_page}", style="info")



import sys
import os
from platform import python_version

# https://github.com/willmcgugan/rich
from rich.console import Console
from rich.theme import Theme

import pydbhub.dbhub as dbhub

if __name__ == '__main__':
    custom_theme = Theme({
        "info": "green",
        "warning": "yellow",
        "error": "bold red"
    })
    console = Console(theme=custom_theme)

    if python_version()[0:3] < '3.7':
        console.print(
            "[ERROR] Make sure you have Python 3.7+ installed, quitting.\n\n", style="error")
        sys.exit(1)

    # Create a new DBHub.io API object
    db = dbhub.Dbhub(config_file=f"{os.path.join(os.path.dirname(__file__), '..', 'config.ini')}")

    # Retrieve the column info for a table or view in the remote database
    table = "table1"
    columns, err = db.Columns(db_owner='justinclift', db_name="Join Testing.sqlite", table=table)
    if err is not None:
        console.print(f"[ERROR] {err}", style="error")
        sys.exit(1)

    # Display the retrieved column details
    console.print(f"Columns on table or view '{table}':", style="info")
    for column in columns:
        console.print(f"   - Name: {column.name}", style="info")
        console.print(f"        Column ID    : {column.column_id}", style="info")
        console.print(f"        Data type    : {column.data_type}", style="info")
        console.print(f"        Defualt value: {column.default_value}", style="info")
        console.print(f"        Not null     : {column.not_null}", style="info")
        console.print(f"        Primary key  : {column.primary_key}", style="info")




import os
import io
import configparser
import base64
import datetime
from typing import List, Tuple, Dict
from dataclasses import dataclass
from typing_extensions import Literal

# https://dateutil.readthedocs.io/
import dateutil.parser as p


import pydbhub.httphub as httphub


# Dictionnary to object
class _DbhubDictToObject(object):
    def __init__(self, data):
        for name, value in data.items():
            setattr(self, name, self._wrap(value))

    def _wrap(self, value):
        if isinstance(value, (tuple, list, set, frozenset)):
            return type(value)([self._wrap(v) for v in value])
        else:
            return _DbhubDictToObject(value) if isinstance(value, dict) else value


# Connection is a simple container holding the API key and address of the DBHub.io server
@dataclass()
class Connection:
    api_key: str = ''
    server: str = "https://api.dbhub.io"


# Identifier holds information used to identify a specific commit, tag, release, or the head of a specific branch
@dataclass()
class Identifier:
    branch: str = ''
    commit_id: str = ''
    release: str = ''
    tag: str = ''


# UploadInformation holds information used when uploading
@dataclass()
class UploadInformation:
    identifier: Identifier = Identifier()
    commitmsg: str = ''
    sourceurl: str = ''
    lastmodified: datetime.datetime = None
    licence: str = ''
    public: bool = True
    force: bool = True
    committimestamp: datetime.datetime = None
    authorname: str = ''
    authoremail: str = ''
    committername: str = ''
    committeremail: str = ''
    otherparents: str = ''
    dbshasum: str = ''


class Dbhub:
    PRESERVE_PK_MERGE = 1
    NEX_PK_MERGE = 2

    def __init__(self, config_data: str = None, config_file: str = None):
        """
        Creates a new DBHub.io connection object.  It doesn't connect to DBHub.io.
        Connection only occurs when subsequent functions (eg Query()) are called.

        Parameters
        ----------
        key : str
            API key
        config_data : str
            INI configuration data from a string
        config_file : str
            INI configuration file
        """

        config = configparser.ConfigParser()
        if config_data:
            config.read_string(config_data)
        elif config_file:
            if os.path.exists(config_file) > 0:
                try:
                    with open(config_file) as f:
                        config.read_file(f)
                except IOError as e:
                    raise ValueError(f"Failed to read config file: {config_file} Erreor: {e}")
            else:
                raise ValueError(f"INI configuration file: {config_file} doesn't exist")
        else:
            raise ValueError("No INI configuration specified")

        if config.has_section('dbhub') is False:
            raise configparser.NoSectionError('dbhub')
        if config.has_option('dbhub', 'api_key') is False:
            raise configparser.NoOptionError('api_key', 'dbhub')
        if config.has_option('dbhub', 'db_owner') is False:
            raise configparser.NoOptionError('db_owner', 'dbhub')
        if config.has_option('dbhub', 'db_name') is False:
            raise configparser.NoOptionError('db_name', 'dbhub')

        self._connection = Connection(api_key=config['dbhub'].get('api_key'))

    def __prepareVals(self, dbOwner: str = None, dbName: str = None, ident: Identifier = None):
        data = {}
        if len(self._connection.api_key) > 0:
            data['apikey'] = (None, self._connection.api_key)
        if dbOwner is not None:
            data['dbowner'] = (None, dbOwner)
        if dbName is not None:
            data['dbname'] = (None, dbName)
        if ident is not None:
            if ident.branch is not None:
                data['branch'] = (None, ident.branch)
            if ident.commit_id is not None:
                data['commit'] = (None, ident.commit_id)
            if ident.release is not None:
                data['release'] = (None, ident.release)
            if ident.tag is not None:
                data['tag'] = (None, ident.tag)
        return data

    def Databases(self) -> Tuple[List[str], str]:
        """
        Returns the list of databases in the requesting users account.
        Ref: https://api.dbhub.io/#databases

        Returns
        -------
        Tuple[List[Dict], str]
            The returned data is
                - a list of string. Each string is the name of a database in your account.
                - a string describe error if occurs
        """
        data = {
            'apikey': (None, self._connection.api_key),
        }
        return httphub.send_request_json(self._connection.server + "/v1/databases", data)

    def Columns(self, db_owner: str, db_name: str, table: str, ident: Identifier = None) -> Tuple[List[Dict], str]:
        """
        Returns the details of all columns in a table or view, as per the SQLite "table_info" PRAGMA.
        Ref: https://api.dbhub.io/#columns

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database
        table : str
            The name of the table or view to return column information for
        ident : Identifier
            Information used to identify a specific commit, tag, release, or the head of a specific branch

        Returns
        -------
        Tuple[List[Dict], str]
            The returned data is
                - a list containing the details of each column in dictionnary objects.
                - a string describe error if occurs
        """
        data = self.__prepareVals(db_owner, db_name, ident)
        data['table'] = table

        res, err = httphub.send_request_json(self._connection.server + "/v1/columns", data)
        if err:
            return None, res

        for i, val in enumerate(res):
            res[i] = _DbhubDictToObject(val)

        return res, None

    def Delete(self, db_name: str) -> str:
        """
        Delete a database from the requesting users account.
        Ref: https://api.dbhub.io/#delete

        Parameters
        ----------
        db_name : str
            The name of the database

        Returns
        -------
        str
            a string describe error if occurs
        """
        data = self.__prepareVals(dbName=db_name)
        res, err = httphub.send_request_json(self._connection.server + "/v1/delete", data)
        if err:
            return res

        return ''

    def Branches(self, db_owner: str, db_name: str) -> Tuple[Dict, str, str]:
        """
        List of branches for a database
        Ref: https://api.dbhub.io/#branches

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[Dict, str, str]
            The returned data is
                - a dicrionnary containing the details for each of the branches
                - a string containing the default branch name
                - a string describe error if occurs
        """
        data = self.__prepareVals(dbOwner=db_owner, dbName=db_name)
        res, err = httphub.send_request_json(self._connection.server + "/v1/branches", data)
        if err:
            return None, None, res

        branches = {}
        for branche_name in res["branches"]:
            branches.update({branche_name: _DbhubDictToObject(res['branches'][branche_name])})

        return branches, res["default_branch"], None

    def Commits(self, db_owner: str, db_name: str) -> Tuple[List[Dict], str]:
        """
        Returns the details of all commits for a database
        Ref: https://api.dbhub.io/#commits

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[Dict, str]
            The returned data is
                - a dictionnary containing the details of all commits in the database
                - a string describe error if occurs
        """
        data = self.__prepareVals(dbOwner=db_owner, dbName=db_name)
        res, err = httphub.send_request_json(self._connection.server + "/v1/commits", data)
        if err:
            return None, res

        commits = [_DbhubDictToObject(res[i]) for i in res]
        for commit in commits:
            commit.timestamp = p.parse(commit.timestamp)
            for entry in commit.tree.entries:
                entry.last_modified = p.parse(entry.last_modified)

        return commits, None

    def Diff(self, db_owner_a: str, db_name_a: str, ident_a: Identifier, db_owner_b: str, db_name_b: str, ident_b: Identifier, merge: Literal) -> Tuple[Dict, str]:
        """
        Generates a diff between two databases or two versions of a database

        Parameters
        ----------
        db_owner_a : str
            The owner of the first database
        db_name_a : str
            The name of the first database
        ident_a : Identifier
            Information used to identify a specific commit, tag, release, or the head of a specific branch
        db_owner_b : str
            The owner of the second database
        db_name_b : str
            The name of the second database
        ident_b : Identifier
            Information used to identify a specific commit, tag, release, or the head of a specific branch
        merge : Literal
            Specifies the type of SQL statements included in the diff results

        Returns
        -------
        Tuple[Dict, str]
            The returned data is
                - a dicrionnary containing the differences between two commits of two databases
                - a string describe error if occurs
        """
        data = {
            'apikey': (None, self._connection.api_key),
            'dbowner_a': db_owner_a,
            'dbname_a': db_name_a,
            'dbowner_b': db_owner_b,
            'dbname_b': db_name_b,
        }

        if ident_a:
            if ident_a.branch:
                data['branch_a'] = ident_a.branch
            if ident_a.commit_id:
                data['commit_a'] = ident_a.commit_id
            if ident_a.release:
                data['release_a'] = ident_a.release
            if ident_a.tag:
                data['tag_a'] = ident_a.tag

        if ident_b:
            if ident_b.branch:
                data['branch_b'] = ident_b.branch
            if ident_b.commit_id:
                data['commit_b'] = ident_b.commit_id
            if ident_b.release:
                data['release_b'] = ident_b.release
            if ident_b.tag:
                data['tag_b'] = ident_b.tag

        if merge == self.PRESERVE_PK_MERGE:
            data['merge'] = 'preserve_pk'
        elif merge == self.NEX_PK_MERGE:
            data['merge'] = 'new_pk'
        else:
            data['merge'] = 'none'

        # Fetch the diffs
        res, err = httphub.send_request_json(self._connection.server + "/v1/diff", data)
        if err:
            return None, res

        return _DbhubDictToObject(res), None

    def Download(self, db_owner: str, db_name: str) -> Tuple[List[bytes], str]:
        """
        Get the requested SQLite database file as a stream of bytes
        Ref: https://api.dbhub.io/#download

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[List[bytes], str]
            The returned data is
                - database file as a list of bytes
                - a string describe error if occurs
        """
        data = self.__prepareVals(db_owner, db_name)
        return httphub.send_request(self._connection.server + "/v1/download", data)

    def Indexes(self, db_owner: str, db_name: str) -> Tuple[List[Dict], str]:
        """
        Returns the details of all indexes in a SQLite database
        Ref: https://api.dbhub.io/#indexes

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[Dict, str]
            The returned data is
                - a dicrionnary containing the details of all indexes in the database
                - a string describe error if occurs
        """
        data = self.__prepareVals(db_owner, db_name)
        res, err = httphub.send_request_json(self._connection.server + "/v1/indexes", data)
        if err:
            return None, res

        indexes = [_DbhubDictToObject(index) for index in res]

        return indexes, None

    def Metadata(self, db_owner: str, db_name: str) -> Tuple[List[Dict], str]:
        """
        Returns the commit, branch, release, tag and web page information for a database
        Ref: https://api.dbhub.io/#metadata

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[Dict, str]
            The returned data is
                - a dictionnary containing the details of all indexes in the database
                - a string describe error if occurs
        """
        data = self.__prepareVals(db_owner, db_name)
        res, err = httphub.send_request_json(self._connection.server + "/v1/metadata", data)
        if err:
            return None, res

        metadata = _DbhubDictToObject(res)

        metadata.branches = {}
        for branche in res["branches"]:
            metadata.branches.update({branche: _DbhubDictToObject(res['branches'][branche])})

        metadata.releases = {}
        for release in res["releases"]:
            metadata.releases.update({release: _DbhubDictToObject(res['releases'][release])})

        metadata.tags = {}
        for tag in res["tags"]:
            metadata.tags.update({tag: _DbhubDictToObject(res['tags'][tag])})

        commits = [_DbhubDictToObject(res['commits'][i]) for i in res['commits']]
        for commit in commits:
            commit.timestamp = p.parse(commit.timestamp)
        metadata.commits = commits

        return metadata, None

    def Query(self, db_owner: str, db_name: str, sql: str) -> Tuple[List, str]:
        """
        Run a SQLite query (SELECT only) on the chosen database, returning the results.
        Ref: https://api.dbhub.io/#query

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database
        sql : str
            The SQLite query (SELECT only)

        Returns
        -------
        Tuple[str, str]
            The returned data is
                - an array of dictionnary returned from the SQL query having three components:
                    - The name of the return field
                    - The type of data in the field (numeric)
                    - The value of the field
                - a string describe error if occurs
        """
        data = self.__prepareVals(db_owner, db_name)
        data['sql'] = base64.b64encode(sql.encode('ascii'))
        res, err = httphub.send_request_json(self._connection.server + "/v1/query", data)
        if err:
            return None, res

        rows = []
        for result_row in res:
            one_row = {}
            for data in result_row:
                result = {
                    0: lambda v: base64.b64decode(v.encode('ascii')) if isinstance(v, str) else None,   # Binary
                    1: lambda v: "",                                                                    # Image - just output as an empty string (for now)
                    2: lambda v: None,                                                                  # Null
                    3: lambda v: str(v) if isinstance(v, str) else "",                                  # Text
                    4: lambda v: int(v),                                                                # Integer
                    5: lambda v: float(v)                                                               # Float
                }[data['Type']](data['Value'])
                one_row.update({data['Name']: result})
            rows.append(one_row)

        return rows, None

    def Releases(self, db_owner: str, db_name: str) -> Tuple[List[Dict], str]:
        """
        Returns the details of all releases for a database
        Ref: https://api.dbhub.io/#releases

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[Dict, str]
            The returned data is
                - a dictionnary containing the details of all the releases of the database
                - a string describe error if occurs
        """
        # Prepare the API parameters
        data = self.__prepareVals(db_owner, db_name)
        # Fetch the releases
        res, err = httphub.send_request_json(self._connection.server + "/v1/releases", data)
        if err:
            return None, res

        releases = {index: _DbhubDictToObject(res[index]) for index in res}
        for release in releases:
            releases[release].date = p.parse(releases[release].date)

        return releases, None

    def Tables(self, db_owner: str, db_name: str) -> Tuple[List[str], str]:
        """
        Returns the list of tables in a SQLite database
        Ref: https://api.dbhub.io/#tables

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[List[str], str]
            The returned data is
                - an string array.  Each string is the name of a table in the database.
                - a string describe error if occurs
        """
        # Prepare the API parameters
        data = self.__prepareVals(db_owner, db_name)
        # Fetch the list of tables
        res, err = httphub.send_request_json(self._connection.server + "/v1/tables", data)
        if err:
            return None, res

        return res, None

    def Tags(self, db_owner: str, db_name: str) -> Tuple[List[Dict], str]:
        """
        Returns the details of all tags for a database
        Ref: https://api.dbhub.io/#tags

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[Dict, str]
            The returned data is
                - a dictionnary containing the details of all the tags in the database
                - a string describe error if occurs
        """
        # Prepare the API parameters
        data = self.__prepareVals(db_owner, db_name)
        # Fetch the releases
        res, err = httphub.send_request_json(self._connection.server + "/v1/tags", data)
        if err:
            return None, res

        tags = {index: _DbhubDictToObject(res[index]) for index in res}
        for tag in tags:
            tags[tag].date = p.parse(tags[tag].date)

        return tags, None

    def Upload(self, db_name: str, info: UploadInformation, db_bytes: io.BufferedReader) -> Tuple[Dict, str]:
        """
        Creates a new database in your account, or adds a new commit to an existing database
        Ref: https://api.dbhub.io/#upload

        Parameters
        ----------
        db_name : str
            The name of the database
        info : UploadInformation
            Upload parameters
        db_bytes : io.BufferedReader
            A buffered binary stream of the database file.

        Returns
        -------
        Tuple[Dict, str]
            The returned data is
                - a dictionnary containing the new commit ID and web page URL
                - a string describe error if occurs
        """
        # Prepare the API parameters
        data = self.__prepareVals(dbName=db_name, ident=info.identifier)

        if info:
            if info.commitmsg:
                data['commitmsg'] = info.commitmsg
            if info.sourceurl:
                data['sourceurl'] = info.sourceurl
            if info.lastmodified:
                data['lastmodified'] = info.committimestamp.astimezone(datetime.timezone.utc).isoformat()
            if info.licence:
                data['licence'] = info.licence
            data['public'] = str(info.public)
            data['force'] = str(info.force)
            if info.committimestamp:
                data['committimestamp'] = info.committimestamp.astimezone(datetime.timezone.utc).isoformat()
            if info.authorname:
                data['authorname'] = info.authorname
            if info.authoremail:
                data['authoremail'] = info.authoremail
            if info.committername:
                data['committername'] = info.committername
            if info.committeremail:
                data['committeremail'] = info.committeremail
            if info.otherparents:
                data['otherparents'] = info.otherparents
            if info.dbshasum:
                data['dbshasum'] = info.dbshasum

        res, err = httphub.send_upload(self._connection.server + "/v1/upload", data, db_bytes)
        if err:
            return None, res

        return res, None

    def Views(self, db_owner: str, db_name: str, ident: Identifier = None) -> Tuple[List[Dict], str]:
        """
        Returns the list of views in a SQLite database
        Ref: https://api.dbhub.io/#views

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database
        ident : Identifier
            Information used to identify a specific commit, tag, release, or the head of a specific branch

        Returns
        -------
        Tuple[List[Dict], str]
            The returned data is
                - a list of string. Each string is the name of a view in the database.
                - a string describe error if occurs
        """
        # Prepare the API parameters
        data = self.__prepareVals(db_owner, db_name, ident)
        # Fetch the list of views
        res, err = httphub.send_request_json(self._connection.server + "/v1/views", data)
        if err:
            return None, res

        return res, None

    def Webpage(self, db_owner: str, db_name: str) -> Tuple[str, str]:
        """
        Returns the address of the database in the webUI. eg. for web browsers.
        Ref: https://api.dbhub.io/#webpage

        Parameters
        ----------
        db_owner : str
            The owner of the database
        db_name : str
            The name of the database

        Returns
        -------
        Tuple[str, str]
            The returned data is
                - a string with the URL of a database
                - a string describe error if occurs
        """
        # Prepare the API parameters
        data = self.__prepareVals(db_owner, db_name)
        # Fetch the address of the database in the webUI
        res, err = httphub.send_request_json(self._connection.server + "/v1/webpage", data)
        if err:
            return None, res

        return res['web_page'], None



import sys
import os
import datetime
from platform import python_version

# https://github.com/willmcgugan/rich
from rich.console import Console
from rich.theme import Theme

import pydbhub.dbhub as dbhub


if __name__ == '__main__':
    custom_theme = Theme({
        "info": "green",
        "warning": "yellow",
        "error": "bold red"
    })
    console = Console(theme=custom_theme)

    if python_version()[0:3] < '3.7':
        console.print(
            "[ERROR] Make sure you have Python 3.7+ installed, quitting.\n\n", style="error")
        sys.exit(1)

    # Create a new DBHub.io API object
    db = dbhub.Dbhub(config_file=f"{os.path.join(os.path.dirname(__file__), '..', 'config.ini')}")

    # Prepare any information you want to include with the upload (eg a commit message, etc)
    info = dbhub.UploadInformation(
        commitmsg="An example upload",
        committimestamp=datetime.datetime.fromisoformat("2021-06-01 10:00:00"),
    )

    try:
        # Read the database file
        myDB = os.path.join(os.getcwd(), "examples", "upload", 'example.db')
        f = open(myDB, 'rb')
    except FileNotFoundError:
        print(f"File {myDB} not found.  Aborting")
        sys.exit(1)
    except OSError:
        print(f"OS error occurred trying to open {myDB}")
        sys.exit(1)
    except Exception as err:
        print(f"Unexpected error opening {myDB} is", repr(err))
        sys.exit(1)
    else:
        with f:
            # Upload the database
            res, err = db.Upload(db_name='somedb.sqlite', info=info, db_bytes=f)
            if err is not None:
                console.print(f"[ERROR] {err}", style="error")
                sys.exit(1)

            console.print(f"Database uploaded, commit: {res['commit']}", style="info")



import sys
import os
from platform import python_version

# https://github.com/willmcgugan/rich
from rich.console import Console
from rich.theme import Theme

import pydbhub.dbhub as dbhub


if __name__ == '__main__':
    custom_theme = Theme({
        "info": "green",
        "warning": "yellow",
        "error": "bold red"
    })
    console = Console(theme=custom_theme)

    if python_version()[0:3] < '3.7':
        console.print("[ERROR] Make sure you have Python 3.7+ installed, quitting.\n\n", style="error")
        sys.exit(1)

    # Create a new DBHub.io API object
    db = dbhub.Dbhub(config_file=f"{os.path.join(os.path.dirname(__file__), '..', 'config.ini')}")

    # Retrieve the list of branches for a database
    commits, err = db.Commits("justinclift", "Join Testing.sqlite")
    if err is not None:
        console.print(f"[ERROR] {err}", style="error")
        sys.exit(1)

    # Display the commits
    console.print('Commits:', style="info")
    for commit in commits:
        console.print(f"   - {commit.id}", style="info")
        if commit.committer_name != '':
            console.print(f"      Committer Name : {commit.committer_name}", style="info")
        if commit.committer_email != '':
            console.print(f"      Committer Email: {commit.committer_email}", style="info")
        console.print(f"      Timestamp      : {commit.timestamp}", style="info")
        console.print(f"      Author Name    : {commit.author_name}", style="info")
        console.print(f"      Author Email   : {commit.author_email}", style="info")
        if commit.message != '':
            console.print(f"      Message        : {commit.message}", style="info")
        if commit.parent == '':
            console.print("      Parent         : NONE", style="info")
        else:
            console.print(f"      Parent         : {commit.parent}", style="info")



import sys
import os
from platform import python_version

# https://github.com/willmcgugan/rich
from rich.console import Console
from rich.theme import Theme

import pydbhub.dbhub as dbhub


if __name__ == '__main__':
    custom_theme = Theme({
        "info": "green",
        "warning": "yellow",
        "error": "bold red"
    })
    console = Console(theme=custom_theme)

    if python_version()[0:3] < '3.7':
        console.print("[ERROR] Make sure you have Python 3.7+ installed, quitting.\n\n", style="error")
        sys.exit(1)

    # Create a new DBHub.io API object
    db = dbhub.Dbhub(config_file=f"{os.path.join(os.path.dirname(__file__), '..', 'config.ini')}")

    # Retrieve the list of branches for a database
    res, default_branch, err = db.Branches("justinclift", "Marine Litter Survey (Keep Northern Ireland Beautiful).sqlite")
    if err is not None:
        console.print(f"[ERROR] {err}", style="error")
        sys.exit(1)

    # Display the retrieved list of branches
    default_branch_text = ""
    console.print('Branches:', style="info")
    for branche_name in res:
        if branche_name == default_branch:
            default_branch_text = ", default branch"
        console.print(f"   - {branche_name} (commits: {res[branche_name].commit_count}{default_branch_text})", style="info")



import pytest
import configparser
import os

import pydbhub.dbhub as dbhub

CONFIG = '''
    [dbhub]
    # dbhub.io Api key
    api_key = YOUR_DBHub.io_API_KEY_Here

    # The owner of the database to query
    db_owner = justinclift

    # The name of the database
    #   - https://dbhub.io/justinclift/Join%20Testing.sqlite
    db_name = Join Testing.sqlite
'''


@pytest.fixture(scope='session')
def connection():
    return dbhub.Dbhub(config_data=CONFIG)


def test_configuration():
    config = configparser.ConfigParser()
    config.read_string(CONFIG)

    assert config.has_section('dbhub')
    assert config.has_option('dbhub', 'api_key')
    assert config.has_option('dbhub', 'db_owner')
    assert config.has_option('dbhub', 'db_name')


# https://api.dbhub.io/#databases
def test_databases(connection):
    databases, err = connection.Databases()
    assert err is None, err
    assert databases is not None, 'No data result'
    assert len(databases) >= 1, 'Missing data result'
    matching = ("Marine Litter Survey (Keep Northern Ireland Beautiful).sqlite") in databases
    matching |= ("Join Testing.sqlite") in databases
    assert matching


# https://api.dbhub.io/#columns
def test_columns(connection):
    columns, err = connection.Columns(db_owner='lemoussel', db_name="Join Testing.sqlite", table='table1')
    assert err is None, err
    assert columns is not None, 'No data result'
    assert len(columns) == 2, 'Missing data result'


# https://api.dbhub.io/#delete
def test_delete(connection):
    err = connection.Delete("Join Testing.sqlite")
    assert err is None, err


# https://api.dbhub.io/#branches
def test_branches(connection):
    res, default_branch, err = connection.Branches("justinclift", "Marine Litter Survey (Keep Northern Ireland Beautiful).sqlite")
    assert err is None, err
    assert default_branch == 'release'
    assert len(res) == 3


# https://api.dbhub.io/#commits
def test_commits(connection):
    commits, err = connection.Commits("justinclift", "Join Testing.sqlite")
    assert err is None, err
    assert len(commits) == 3
    assert commits[0].id == '7beb90a62a842dcb095592a5083f22533552da17eb72891d26c87ae48070885d'
    assert commits[1].author_name == 'Justin Clift'
    assert commits[2].parent == ''


# https://api.dbhub.io/#diff
def test_diff(connection):
    commit1 = dbhub.Identifier(commit_id="34cbeebfc347a09406707f4220cd40f60778692523d2e7d227ccd92f4125c9ea")
    commit2 = dbhub.Identifier(commit_id="bc6a07955811d86db79e9b4f7fdc3cb2360d40da793066510d792588a8bf8de2")

    diffs, err = connection.Diff("justinclift", "DB4S download stats.sqlite", commit1, '', '', commit2, connection.PRESERVE_PK_MERGE)
    assert err is None, err
    assert hasattr(diffs, 'diff')
    assert hasattr(diffs.diff[0], 'data')


# https://api.dbhub.io/#download
def test_download(connection):
    buf, err = connection.Download(db_name="Join Testing.sqlite", db_owner="justinclift")
    assert err is None, err
    assert len(buf) == 12288


# https://api.dbhub.io/#indexes
def test_indexes(connection):
    indexes, err = connection.Indexes(db_name="DB4S daily users by country.sqlite", db_owner="justinclift")
    assert err is None, err
    assert len(indexes) == 1
    assert indexes[0].name == 'active_users-date_idx'
    assert indexes[0].table == 'active_users'


# https://api.dbhub.io/#query
def test_query(connection):
    result, err = connection.Query(
        db_owner="justinclift",
        db_name="Join Testing.sqlite",
        sql='''
        SELECT table1.id, table1.name, table2.value
        FROM table1 JOIN table2
        USING (id)
        ORDER BY table1.id
        '''
    )
    assert err is None, err
    assert len(result) == 6
    assert result[0]['Name'] == 'Foo'
    assert result[0]['value'] == 5.1
    assert result[5]['Name'] == 'Batty'
    assert result[5]['value'] == 3


# https://api.dbhub.io/#indexes
def test_releases(connection):
    releases, err = connection.Releases(db_name="Join Testing.sqlite", db_owner="justinclift")
    releases_name = list(releases.keys())
    assert err is None, err
    assert len(releases_name) == 2
    assert releases_name[0] == 'v0.0.1'
    assert releases[releases_name[0]].description == "A first testing release :)"


# https://api.dbhub.io/#tables
def test_tables(connection):
    tables, err = connection.Tables(db_name="Join Testing.sqlite", db_owner="justinclift")
    assert err is None, err
    assert len(tables) == 2
    assert tables[0] == 'table1'


# https://api.dbhub.io/#tags
def test_tags(connection):
    tags, err = connection.Tags(db_name="Join Testing.sqlite", db_owner="justinclift")
    tags_name = list(tags.keys())
    assert err is None, err
    assert len(tags_name) == 2
    assert tags_name[0] == 'anothertag'
    assert tags[tags_name[0]].description == "A second tag on this database, also for demo purposes."


# https://api.dbhub.io/#upload
def test_upload(connection):
    info = dbhub.UploadInformation(
        commitmsg="An test upload",
    )
    my_db = os.path.join(os.getcwd(), 'tests', 'example.db')
    with open(my_db, 'rb') as f:
        res, err = connection.Upload(db_name='test_somedb.sqlite', info=info, db_bytes=f)
    assert err is None, err


# https://api.dbhub.io/#views
def test_views(connection):
    views, err = connection.Views(db_name="Join Testing.sqlite", db_owner="justinclift", ident=dbhub.Identifier(branch="master"))
    assert err is None, err
    assert len(views) == 1
    assert views[0] == 'joinedView'


# https://api.dbhub.io/#webpage
def test_webpage(connection):
    webpage, err = connection.Webpage("justinclift", "Join Testing.sqlite")
    assert err is None, err
    assert webpage == 'https://dbhub.io/justinclift/Join Testing.sqlite'



import setuptools

import pydbhub

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='pydbhub',
    version=pydbhub.__version__,
    author="LeMoussel",
    author_email="cnhx27@gmail.com",
    license='MIT',
    url="https://github.com/LeMoussel/pydbhub",
    description='A Python library for accessing and using SQLite databases on DBHub.io',
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir={"": "pydbhub"},
    packages=setuptools.find_packages(where="pydbhub"),
    install_requires=[
        'requests',
        'python_dateutil',
        'rich'
    ],
    python_requires='>=3.7',
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import pydbhub
from typing import Any, Dict, List, Tuple
from json.decoder import JSONDecodeError
import requests
import io


def send_request_json(query_url: str, data: Dict[str, Any]) -> Tuple[List[Any], str]:
    """
    send_request_json sends a request to DBHub.io, formatting the returned result as JSON

    Parameters
    ----------
    query_url : str
        url of the API endpoint
    data : Dict[str, Any]
        data to be processed to the server.

    Returns
    -------
    Tuple[List[Any], str]
    The returned data is
        - a list of JSON object.
        - a string describe error if occurs
    """

    try:
        headers = {'User-Agent': f'pydbhub v{pydbhub.__version__}'}
        response = requests.post(query_url, data=data, headers=headers)
        response.raise_for_status()
        return response.json(), None
    except JSONDecodeError as e:
        return None, e.args[0]
    except TypeError as e:
        return None, e.args[0]
    except requests.exceptions.HTTPError as e:
        try:
            return response.json(), e.args[0]
        except JSONDecodeError:
            return None, e.args[0]
    except requests.exceptions.RequestException as e:
        cause = e.args(0)
        return None, str(cause.args[0])


def send_request(query_url: str, data: Dict[str, Any]) -> Tuple[List[bytes], str]:
    """
    send_request sends a request to DBHub.io.

    Parameters
    ----    query_url : str
        url of the API endpoint
    data : Dict[str, Any]
        data to be processed to the server.------


    Returns
    -------
    List[bytes]
        database file is returned as a list of bytes
    """
    try:
        headers = {'User-Agent': f'pydbhub v{pydbhub.__version__}'}
        response = requests.post(query_url, data=data, headers=headers)
        response.raise_for_status()
        return response.content, None
    except requests.exceptions.HTTPError as e:
        return None, e.args[0]
    except requests.exceptions.RequestException as e:
        cause = e.args(0)
        return None, str(cause.args[0])


def send_upload(query_url: str, data: Dict[str, Any], db_bytes: io.BufferedReader) -> Tuple[List[Any], str]:
    """
    send_upload uploads a database to DBHub.io.

    Parameters
    ----------
    query_url : str
        url of the API endpoint.
    data : Dict[str, Any]
        data to be processed to the server.
    db_bytes : io.BufferedReader
        A buffered binary stream of the database file.

    Returns
    -------
    Tuple[List[Any], str]
    The returned data is
        - a list of JSON object.
        - a string describe error if occurs
    """
    try:
        headers = {'User-Agent': f'pydbhub v{pydbhub.__version__}'}
        files = {"file": db_bytes}
        response = requests.post(query_url, data=data, headers=headers, files=files)
        response.raise_for_status()
        if response.status_code != 201:
            # The returned status code indicates something went wrong
            try:
                return response.json(), str(response.status_code)
            except JSONDecodeError:
                return None, str(response.status_code)
        return response.json(), None
    except requests.exceptions.HTTPError as e:
        try:
            return response.json(), e.args[0]
        except JSONDecodeError:
            return None, e.args[0]
    except requests.exceptions.RequestException as e:
        cause = e.args(0)
        return None, str(cause.args[0])



import sys
import os
from platform import python_version

# https://github.com/willmcgugan/rich
from rich.console import Console
from rich.theme import Theme

import pydbhub.dbhub as dbhub


if __name__ == '__main__':
    custom_theme = Theme({
        "info": "green",
        "warning": "yellow",
        "error": "bold red"
    })
    console = Console(theme=custom_theme)

    if python_version()[0:3] < '3.7':
        console.print("[ERROR] Make sure you have Python 3.7+ installed, quitting.\n\n", style="error")
        sys.exit(1)

    # Create a new DBHub.io API object
    db = dbhub.Dbhub(config_file=f"{os.path.join(os.path.dirname(__file__), '..', 'config.ini')}")

    # Retrieve the tags for the remote database
    tags, err = db.Tags("justinclift", "Join Testing.sqlite")
    if err is not None:
        console.print(f"[ERROR] {err}", style="error")
        sys.exit(1)

    # Display tags
    console.print('Tags:', style="info")
    for tag_name in tags:
        console.print(f"   - {tag_name}", style="info")
        console.print(f"       Commit: {tags[tag_name].commit}", style="info")
        console.print(f"       Date: {tags[tag_name].date}", style="info")
        console.print(f"       Tagger Name: {tags[tag_name].name}", style="info")
        console.print(f"       Tagger Email: {tags[tag_name].email}", style="info")
        console.print(f"       Description: {tags[tag_name].description}", style="info")



import sys
import os
from platform import python_version

# https://github.com/willmcgugan/rich
from rich.console import Console
from rich.theme import Theme


import pydbhub.dbhub as dbhub

if __name__ == '__main__':
    custom_theme = Theme({
        "info": "green",
        "warning": "yellow",
        "error": "bold red"
    })
    console = Console(theme=custom_theme)

    if python_version()[0:3] < '3.7':
        console.print("[ERROR] Make sure you have Python 3.7+ installed, quitting.\n\n", style="error")
        sys.exit(1)

    # Create a new DBHub.io API object
    db = dbhub.Dbhub(config_file=f"{os.path.join(os.path.dirname(__file__), '..', 'config.ini')}")

    # Retrieve the differences between two commmits of the same database
    commit1 = dbhub.Identifier(commit_id="34cbeebfc347a09406707f4220cd40f60778692523d2e7d227ccd92f4125c9ea")
    commit2 = dbhub.Identifier(commit_id="bc6a07955811d86db79e9b4f7fdc3cb2360d40da793066510d792588a8bf8de2")

    diffs, err = db.Diff("justinclift", "DB4S download stats.sqlite", commit1, '', '', commit2, db.PRESERVE_PK_MERGE)
    if err is not None:
        console.print(f"[ERROR] {err}", style="error")
        sys.exit(1)

    # Display the SQL statements needed to turn the first version of the database into the second.
    # This should produce a similar output to the sqldiff utility.
    for diff in diffs.diff:
        # Print schema changes to this object if there are any
        if hasattr(diff, 'schema'):
            console.print(diff.schema.sql, style="info")

        # Loop over all data changes in this object if there are any
        for data in diff.data:
            console.print(data.sql, style="info")

    '''
    for diff in diffs['diff']:
        # Print schema changes to this object if there are any
        if 'schema' in diff and diff['schema']:
            console.print(diff['schema']['sql'], style="info")

        # Loop over all data changes in this object if there are any
        for data in diff['data']:
            console.print(data['sql'], style="info")
    '''
